
from playwright.sync_api import sync_playwright
import os,time
from datetime import datetime
from celery import Celery
from kombu import Queue
import time
import time
import random
from playwright.sync_api import sync_playwright
from playwright.sync_api import Route, Response
from celery import shared_task
import requests
from celery import shared_task,chord, group, signature, uuid
from celery.signals import (
    after_setup_task_logger,
    task_success,
    task_prerun,
    task_postrun,
    celeryd_after_setup,
    celeryd_init,
)

targetUrl = "https://gamerushbin.blogspot.com/2022/12/the-top-10-best-meal-kit-delivery.html";
fakeHtml="""<html><body>
  <h1>fake html</h1>
  <iframe data-aa='2144138' src='//ad.a-ads.com/2144138?size=468x60' style='width:468px; height:60px; border:0px; padding:0; overflow:hidden; background-color: transparent;'></iframe>
</body></html>
"""

@shared_task(bind=True)
def adEmuAAD():
    # 随机延时 搭配 定时任务固定的触发间隔，能做到看起来时间点随机。
    time.sleep(random.randint(0, 15))
    def handle_route_target_url(route: Route):
        route.fulfill(status=200, content_type="text/html", body=fakeHtml)
    """使用住宅ip浏览a-ad广告"""
    with sync_playwright() as p:
        browser = p.chromium.launch(
            headless=False,
            proxy={
                "server": "http://@geo.iproyal.com:12321",
                "username": "mattwin",
                "password": "feihuo321_country-us_skipispstatic-1"
            },
            timeout=60*1000
        )
        context = browser.new_context(
            http_credentials={"username": "bill", "password": "pa55w0rd"}
        )
        page = context.new_page()
        
        page.route(targetUrl, handle_route_target_url)
        page.goto(targetUrl)
        time.sleep(60)
        browser.close()
@shared_task
def mtxad_debug():
    print("mtxad debug1-------------", flush=True)
    return "some result for mtxad_debug"



# Copyright 2018 Regents of the University of Michigan

# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at http://www.apache.org/licenses/LICENSE-2.0

# import mongoengine
import traceback
import datetime

from celery import schedules
# from celerybeatmongo.models import PeriodicTask
from celery.beat import Scheduler, ScheduleEntry
from celery.utils.log import get_logger
from celery import current_app


logger = get_logger(__name__)


class MongoScheduleEntry(ScheduleEntry):

    def __init__(self, task):
        self._task = task

        self.app = current_app._get_current_object()
        self.name = self._task.name
        self.task = self._task.task

        self.schedule = self._task.schedule

        self.args = self._task.args
        self.kwargs = self._task.kwargs
        self.options = {
            'queue': self._task.queue,
            'exchange': self._task.exchange,
            'routing_key': self._task.routing_key,
            'expires': self._task.expires,
            'soft_time_limit': self._task.soft_time_limit,
            'enabled': self._task.enabled
        }
        if self._task.total_run_count is None:
            self._task.total_run_count = 0
        self.total_run_count = self._task.total_run_count

        if not self._task.last_run_at:
            self._task.last_run_at = self._default_now()
        self.last_run_at = self._task.last_run_at

    def _default_now(self):
        return self.app.now()

    def next(self):
        self._task.last_run_at = self.app.now()
        self._task.total_run_count += 1
        self._task.run_immediately = False
        return self.__class__(self._task)

    __next__ = next

    def is_due(self):
        if not self._task.enabled:
            return schedules.schedstate(False, 5.0)   # 5 second delay for re-enable.
        if hasattr(self._task, 'start_after') and self._task.start_after:
            if datetime.datetime.now() < self._task.start_after:
                return schedules.schedstate(False, 5.0)
        if hasattr(self._task, 'max_run_count') and self._task.max_run_count:
            if (self._task.total_run_count or 0) >= self._task.max_run_count:
                self._task.enabled = False
                self._task.save()
                # Don't recheck
                return schedules.schedstate(False, None)
        if self._task.run_immediately:
            # figure out when the schedule would run next anyway
            _, n = self.schedule.is_due(self.last_run_at)
            return True, n
        return self.schedule.is_due(self.last_run_at)

    def __repr__(self):
        return (u'<{0} ({1} {2}(*{3}, **{4}) {{5}})>'.format(
            self.__class__.__name__,
            self.name, self.task, self.args,
            self.kwargs, self.schedule,
        ))

    def reserve(self, entry):
        new_entry = Scheduler.reserve(self, entry)
        return new_entry

    def save(self):
        if self.total_run_count > self._task.total_run_count:
            self._task.total_run_count = self.total_run_count
        if self.last_run_at and self._task.last_run_at and self.last_run_at > self._task.last_run_at:
            self._task.last_run_at = self.last_run_at
        self._task.run_immediately = False
        try:
            self._task.save(save_condition={})
        except Exception:
            logger.error(traceback.format_exc())


class MtxApiScheduler(Scheduler):

    #: how often should we sync in schedule information
    #: from the backend mongo database
    UPDATE_INTERVAL = datetime.timedelta(seconds=5)

    Entry = MongoScheduleEntry

    # Model = PeriodicTask

    def __init__(self, app, *args, **kwargs):
        if hasattr(app.conf, "mongodb_scheduler_db"):
            db = app.conf.get("mongodb_scheduler_db")
        elif hasattr(app.conf, "CELERY_MONGODB_SCHEDULER_DB"):
            db = app.conf.CELERY_MONGODB_SCHEDULER_DB
        else:
            db = "celery"
        if hasattr(app.conf, "mongodb_scheduler_connection_alias"):
            alias = app.conf.get('mongodb_scheduler_connection_alias')
        elif hasattr(app.conf, "CELERY_MONGODB_SCHEDULER_CONNECTION_ALIAS"):
            alias = app.conf.CELERY_MONGODB_SCHEDULER_CONNECTION_ALIAS
        else:
            alias = "default"

        if hasattr(app.conf, "mongodb_scheduler_url"):
            host = app.conf.get('mongodb_scheduler_url')
        elif hasattr(app.conf, "CELERY_MONGODB_SCHEDULER_URL"):
            host = app.conf.CELERY_MONGODB_SCHEDULER_URL
        else:
            host = None

        # self._mongo = mongoengine.connect(db, host=host, alias=alias)

        # if host:
        #     logger.info("backend scheduler using %s/%s:%s",
        #                 host, db, self.Model._get_collection().name)
        # else:
        #     logger.info("backend scheduler using %s/%s:%s",
        #                 "mongodb://localhost", db, self.Model._get_collection().name)
        self._schedule = {}
        self._last_updated = None
        Scheduler.__init__(self, app, *args, **kwargs)
        self.max_interval = (kwargs.get('max_interval')
                             or self.app.conf.CELERYBEAT_MAX_LOOP_INTERVAL or 5)

    def setup_schedule(self):
        print("setup_schedule")
        pass

    def requires_update(self):
        """check whether we should pull an updated schedule
        from the backend database"""
        if not self._last_updated:
            return True
        return self._last_updated + self.UPDATE_INTERVAL < datetime.datetime.now()

    def get_from_database(self):
        self.sync()
        d = {}
        # for doc in self.Model.objects.filter(enabled=True):
        #     d[doc.name] = self.Entry(doc)
        # return d
        
        d['aaa'] = self.Entry(name="dd")
        # d['aaa']  = self.Entry({
        #   # "add-every-3-seconds":{
        #   #     "task": 'mtworker.tasks.test',
        #   #     "schedule": 3.0,
        #   #     "args": ['arg111222333'],
        #   # },
        #   "name":"aaff"
        # }
          
        # )
        # return {
        #           'add-every-3-seconds': {
        # "task": 'mtworker.tasks.test',
        # "schedule": 3.0,
        # "args": ['arg111222333'],
        # },
        # }

    @property
    def schedule(self):
        if self.requires_update():
            self._schedule = self.get_from_database()
            self._last_updated = datetime.datetime.now()
        return self._schedule


    def sync(self):
        logger.debug('Writing entries...')
        print("234234")
        print(self._schedule.values())
        # for entry in self._schedule.values():
        #     entry.save()



#!/usr/bin/env python3
import sys, os, time
import typer
from datetime import datetime
from celery import Celery
from kombu import Queue
import requests
from celery import shared_task,chord, group, signature, uuid
from celery.signals import (
    after_setup_task_logger,
    task_success,
    task_prerun,
    task_postrun,
    celeryd_after_setup,
    celeryd_init,
)
from celery.utils.log import get_task_logger
from celery import Celery
import httpx
from typing import Optional
import logging
from threading import Thread
import json
logger = get_task_logger(__name__)

# celery 原始启动命令参考：
# `celery -A mtxcms worker -l INFO`
# `celery -A mtxcms worker -l INFO -Q mtx_cloud`
# 这个命令的含义是：执行celery命令，从mtxcms模块中找到celery模块，并运行为工作进程。
app = typer.Typer()

def create_celery_app(api_url:str):
    response = httpx.get(api_url)
    print(f'params: {response.json()}')
    param = response.json()
    os.environ.setdefault("MTX_CELERY_BROKER",param["celery_broker"] )
    broker = param["celery_broker"]
    schedule = param['schedule']
    celery_app = Celery('tasks', broker=broker, result_backend=broker)
    # celery_app.conf.beat_schedule = {
    #     'add-every-3-seconds': {
    #         'task': 'mtworker.tasks.test',
    #         'schedule': 3.0,
    #         'args': ('test task--- args',)
    #     },
    # }
    
    # api 后端返回的计划任务配置（可以动态更改）
    celery_app.conf['CELERYBEAT_SCHEDULE'] = schedule
    
    return celery_app

@app.command()
def worker(api: Optional[str] = None,beat: str=None):
    os.environ.setdefault("MTX_WORKER_API_URL",api)
    from mtworker.tasks import app as celery_app
    
    response = httpx.get(api)
    print(f'params: {response.json()}')
    param = response.json()
    os.environ.setdefault("MTX_CELERY_BROKER",param["celery_broker"] )
    broker = param["celery_broker"]
    schedule = param['schedule']
    celery_app.conf.broker_url = broker
    celery_app.conf.result_backend = broker
    print(f'schedule {schedule}')
    # print(celery_app.conf)
    # celery_app.conf.result_backend_url = broker
    # celery_app = Celery('tasks', broker=broker, result_backend=broker)
    
    celery_app.conf.beat_schedule = schedule

    # 导入任务
    import mtworker.tasks.ad_tasks
    if beat:
        # celery_app.start(['beat','-l','DEBUG', '--scheduler', 'mtworker.schedulers:MtxApiScheduler'])
        celery_app.start(['worker','--beat','-l','DEBUG'])
    else:
        celery_app.start(['worker','-l','DEBUG'])
    
def cli():
    app()

if __name__ == "__main__":
    app()
